package com.loafle.commons.server.socket.handler.codec;

import io.netty.buffer.ByteBuf;
import io.netty.buffer.Unpooled;

/**
 * Socket frame containing binary data
 */
public class BinarySocketFrame extends SocketFrame {

  /**
   * Creates a new empty binary frame.
   */
  public BinarySocketFrame() {
    super(Unpooled.buffer(0));
  }

  /**
   * Creates a new binary frame with the specified binary data. The final fragment flag is set to true.
   *
   * @param binaryData
   *            the content of the frame.
   */
  public BinarySocketFrame(ByteBuf binaryData) {
    super(binaryData);
  }

  /**
   * Creates a new binary frame with the specified binary data and the final fragment flag.
   *
   * @param finalFragment
   *            flag indicating if this frame is the final fragment
   * @param rsv
   *            reserved bits used for protocol extensions
   * @param binaryData
   *            the content of the frame.
   */
  public BinarySocketFrame(boolean finalFragment, int rsv, ByteBuf binaryData) {
    super(finalFragment, rsv, binaryData);
  }

  @Override
  public BinarySocketFrame copy() {
    return (BinarySocketFrame) super.copy();
  }

  @Override
  public BinarySocketFrame duplicate() {
    return (BinarySocketFrame) super.duplicate();
  }

  @Override
  public BinarySocketFrame retainedDuplicate() {
    return (BinarySocketFrame) super.retainedDuplicate();
  }

  @Override
  public BinarySocketFrame replace(ByteBuf content) {
    return new BinarySocketFrame(isFinalFragment(), rsv(), content);
  }

  @Override
  public BinarySocketFrame retain() {
    super.retain();
    return this;
  }

  @Override
  public BinarySocketFrame retain(int increment) {
    super.retain(increment);
    return this;
  }

  @Override
  public BinarySocketFrame touch() {
    super.touch();
    return this;
  }

  @Override
  public BinarySocketFrame touch(Object hint) {
    super.touch(hint);
    return this;
  }
}