package com.loafle.commons.server.socket.handler.codec;

import io.netty.buffer.ByteBuf;
import io.netty.buffer.Unpooled;

/**
 * Socket frame containing binary data
 */
public class PingSocketFrame extends SocketFrame {

  /**
   * Creates a new empty ping frame.
   */
  public PingSocketFrame() {
    super(true, 0, Unpooled.buffer(0));
  }

  /**
   * Creates a new ping frame with the specified binary data.
   *
   * @param binaryData
   *            the content of the frame.
   */
  public PingSocketFrame(ByteBuf binaryData) {
    super(binaryData);
  }

  /**
   * Creates a new ping frame with the specified binary data
   *
   * @param finalFragment
   *            flag indicating if this frame is the final fragment
   * @param rsv
   *            reserved bits used for protocol extensions
   * @param binaryData
   *            the content of the frame.
   */
  public PingSocketFrame(boolean finalFragment, int rsv, ByteBuf binaryData) {
    super(finalFragment, rsv, binaryData);
  }

  @Override
  public PingSocketFrame copy() {
    return (PingSocketFrame) super.copy();
  }

  @Override
  public PingSocketFrame duplicate() {
    return (PingSocketFrame) super.duplicate();
  }

  @Override
  public PingSocketFrame retainedDuplicate() {
    return (PingSocketFrame) super.retainedDuplicate();
  }

  @Override
  public PingSocketFrame replace(ByteBuf content) {
    return new PingSocketFrame(isFinalFragment(), rsv(), content);
  }

  @Override
  public PingSocketFrame retain() {
    super.retain();
    return this;
  }

  @Override
  public PingSocketFrame retain(int increment) {
    super.retain(increment);
    return this;
  }

  @Override
  public PingSocketFrame touch() {
    super.touch();
    return this;
  }

  @Override
  public PingSocketFrame touch(Object hint) {
    super.touch(hint);
    return this;
  }
}