package com.loafle.commons.server.socket.handler.codec;

import io.netty.buffer.ByteBuf;
import io.netty.buffer.Unpooled;

/**
 * Socket frame containing binary data
 */
public class PongSocketFrame extends SocketFrame {

  /**
   * Creates a new empty pong frame.
   */
  public PongSocketFrame() {
    super(Unpooled.buffer(0));
  }

  /**
   * Creates a new pong frame with the specified binary data.
   *
   * @param binaryData
   *            the content of the frame.
   */
  public PongSocketFrame(ByteBuf binaryData) {
    super(binaryData);
  }

  /**
   * Creates a new pong frame with the specified binary data
   *
   * @param finalFragment
   *            flag indicating if this frame is the final fragment
   * @param rsv
   *            reserved bits used for protocol extensions
   * @param binaryData
   *            the content of the frame.
   */
  public PongSocketFrame(boolean finalFragment, int rsv, ByteBuf binaryData) {
    super(finalFragment, rsv, binaryData);
  }

  @Override
  public PongSocketFrame copy() {
    return (PongSocketFrame) super.copy();
  }

  @Override
  public PongSocketFrame duplicate() {
    return (PongSocketFrame) super.duplicate();
  }

  @Override
  public PongSocketFrame retainedDuplicate() {
    return (PongSocketFrame) super.retainedDuplicate();
  }

  @Override
  public PongSocketFrame replace(ByteBuf content) {
    return new PongSocketFrame(isFinalFragment(), rsv(), content);
  }

  @Override
  public PongSocketFrame retain() {
    super.retain();
    return this;
  }

  @Override
  public PongSocketFrame retain(int increment) {
    super.retain(increment);
    return this;
  }

  @Override
  public PongSocketFrame touch() {
    super.touch();
    return this;
  }

  @Override
  public PongSocketFrame touch(Object hint) {
    super.touch(hint);
    return this;
  }
}